//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
/*++
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Module Name:  

Abstract:  
   This file implements the NK kernel interfaces for firmware interrupt
  support on the CEPC.
 
Functions:

--*/

#include <windows.h>
#include <oal.h>
#include <nkintr.h>
#include <x86boot.h>

#include "pehdr.h"
#include "romldr.h"
#include "romxip.h"


#define NOT_FIXEDUP ((DWORD*)-1)

//------------------------------------------------------------------------------
//
// dwOEMDrWatsonSize
//
// Global variable which specify DrWatson buffer size. It can be fixed
// in config.bib via FIXUPVAR.
//
#define DR_WATSON_SIZE_NOT_FIXEDUP  ((DWORD)-1)
DWORD dwOEMDrWatsonSize = DR_WATSON_SIZE_NOT_FIXEDUP;

void  PCIInitBusInfo (void);
DWORD OEMPowerManagerInit(void);
void  InitClock(void);
void  x86InitMemory (void);
void  x86InitPICs(void);
void  x86RebootInit(void);
void  RTCPostInit();
void  QPCPostInit();

extern LPCWSTR g_pPlatformManufacturer;
extern LPCWSTR g_pPlatformName;
static BOOL g_fPostInit;
void OALMpInit (void);

// function: disable SATA IRQ
// 1 find SATA device
// 2 move to address: 0x3c
// 3 disable IRQ ([0x3c]=0xff)
void DisableSataIrq();

void DisableEHCILegacy();
void DisableUHCILegacy();




//------------------------------------------------------------------------------
//
// x86InitRomChain
//
// Initialize the rom chain
//
static void x86InitRomChain()
{
    // Added for MultiXIP stuff
    static      ROMChain_t  s_pNextRom[MAX_ROM] = {0};
    DWORD       dwRomCount = 0;
    DWORD       dwChainCount = 0;
    DWORD *     pdwCurXIP;
    DWORD       dwNumXIPs;
    PXIPCHAIN_ENTRY pChainEntry = NULL;
    static DWORD *pdwXIPLoc = NOT_FIXEDUP;

#ifdef DEBUG
    TCHAR       szXIPName[XIP_NAMELEN];
    int         i;
#endif

#ifdef DEBUG
    NKOutputDebugString (TEXT("Looking for rom chain\n"));
#endif

    if(pdwXIPLoc == NOT_FIXEDUP)
    {
#ifdef DEBUG
        NKOutputDebugString (TEXT("Rom chain NOT found\n"));
#endif
        return;  // no chain or not fixed up properly
    }

#ifdef DEBUG
    NKOutputDebugString (TEXT("Rom chain found\n"));
#endif


    // set the top bit to mark it as a virtual address
    pdwCurXIP = (DWORD*)(((DWORD)pdwXIPLoc) | 0x80000000);

    // first DWORD is number of XIPs
    dwNumXIPs = (*pdwCurXIP);

    if(dwNumXIPs > MAX_ROM)
    {
        NKOutputDebugString (TEXT("ERROR: Number of XIPs exceeds MAX\n"));
        return;
    }

    pChainEntry = (PXIPCHAIN_ENTRY)(pdwCurXIP + 1);

    while(dwChainCount < dwNumXIPs)
    {
        if ((pChainEntry->usFlags & ROMXIP_OK_TO_LOAD) &&  // flags indicates valid XIP
            *(const DWORD*)(((DWORD)(pChainEntry->pvAddr)) + ROM_SIGNATURE_OFFSET) == ROM_SIGNATURE)
        {
            s_pNextRom[dwRomCount].pTOC = *(ROMHDR **)(((DWORD)(pChainEntry->pvAddr)) + ROM_SIGNATURE_OFFSET + 4);
            s_pNextRom[dwRomCount].pNext = NULL;

#ifdef DEBUG
            NKOutputDebugString ( _T("XIP found: ") );

            for (i = 0; (i< (XIP_NAMELEN-1)) && (pChainEntry->szName[i] != '\0'); ++i)
            {
                szXIPName[i] = (TCHAR)pChainEntry->szName[i];
            }
            szXIPName[i] = TEXT('\0');
            NKOutputDebugString ( szXIPName );

            NKOutputDebugString ( _T("\n") );
#endif

            if (dwRomCount != 0)
            {
                s_pNextRom[dwRomCount-1].pNext = &s_pNextRom[dwRomCount];
            }
            else
            {
                OEMRomChain = s_pNextRom;
            }
            dwRomCount++;
        }
        else
        {
            NKOutputDebugString ( _T("Invalid XIP found\n") );
        }

        ++pChainEntry;
        dwChainCount++;
    }

#ifdef DEBUG
    {
        ROMChain_t         *pchain = OEMRomChain;

        NKOutputDebugString ( _T("chain contents...\n") );
        while(pchain){
            NKOutputDebugString ( _T("found item\n") );
            pchain = pchain->pNext;
        }
    }
#endif
    
}

//------------------------------------------------------------------------------
//
// OEMInit
//
// OEMInit is called by the kernel after it has performed minimal
// initialization. Interrupts are disabled and the kernel is not
// ready to handle exceptions. The only kernel service available
// to this function is HookInterrupt. This should be used to
// install ISR's for all the hardware interrupts to be handled by
// the firmware. Note that ISR's must be installed for any interrupt
// that is to be routed to a device driver - otherwise the
// InterruptInitialize call from the driver will fail.
//
void OEMInit()
{   
    // Set up the debug zones according to the fix-up variable initialOALLogZones
    OALLogSetZones(initialOALLogZones);

    OALMSG(OAL_FUNC, (L"+OEMInit\r\n"));

    // initialize interrupts
    OALIntrInit ();

    // initialize PIC
    x86InitPICs();

    // Initialize PCI bus information
    PCIInitBusInfo ();
    
    // starts KITL (will be a no-op if KITLDLL doesn't exist)
    KITLIoctl (IOCTL_KITL_STARTUP, NULL, 0, NULL, 0, NULL);

    RETAILMSG(1, (TEXT("[VIA BSP: x86] BSP version: v7.0.3 for WINCE7.0 %s %s"), TEXT(__DATE__), TEXT(__TIME__)));

#ifdef DEBUG

    // Instead of calling OEMWriteDebugString directly, call through exported
    // function pointer.  This will allow these messages to be seen if debug
    // message output is redirected to Ethernet or the parallel port.  Otherwise,
    // lpWriteDebugStringFunc == OEMWriteDebugString.    
    NKOutputDebugString (TEXT("Firmware Init\r\n"));

#endif
    // sets the global platform manufacturer name and platform name
    g_oalIoCtlPlatformManufacturer = g_pPlatformManufacturer;
    g_oalIoCtlPlatformName = g_pPlatformName;

    OEMPowerManagerInit();
    
    // initialize clock
    InitClock();


    
   
    //DisableEHCILegacy();
    //DisableUHCILegacy();   
    //DisableSataIrq(); 

    
    // initialize memory (detect extra ram, MTRR/PAT etc.)
    x86InitMemory ();

    // Reserve 128kB memory for Watson Dumps
    dwNKDrWatsonSize = 0;
    if (dwOEMDrWatsonSize != DR_WATSON_SIZE_NOT_FIXEDUP) {
        dwNKDrWatsonSize = dwOEMDrWatsonSize;
    }

    x86RebootInit();
    
    x86InitRomChain();
    
    OALMpInit ();
 
#ifdef DEBUG
    NKOutputDebugString (TEXT("Firmware Init Done.\r\n"));
#endif

    OALMSG(OAL_FUNC, (L"-OEMInit\r\n"));
}

//------------------------------------------------------------------------------
//
// IsAfterPostInit
//
// TRUE once the kernel has performed a more full featured init. Critical Sections can now be used
//
BOOL IsAfterPostInit() 
{ 
    return g_fPostInit; 
}

//------------------------------------------------------------------------------
//
// x86IoCtlPostInit
//
// Called once the kernel has performed a more full featured init. Critical Sections can now be used
//
BOOL x86IoCtlPostInit(
                      UINT32 code, 
                      __in_bcount(nInBufSize) const void * lpInBuf, 
                      UINT32 nInBufSize, 
                      __out_bcount(nOutBufSize) const void * lpOutBuf, 
                      UINT32 nOutBufSize, 
                      __out UINT32 * lpBytesReturned
                      ) 
{
    UNREFERENCED_PARAMETER(code);
    UNREFERENCED_PARAMETER(lpInBuf);
    UNREFERENCED_PARAMETER(nInBufSize);
    UNREFERENCED_PARAMETER(lpOutBuf);
    UNREFERENCED_PARAMETER(nOutBufSize);

    if (lpBytesReturned) 
    {
        *lpBytesReturned = 0;
    }

    RTCPostInit();
    QPCPostInit();

    g_fPostInit = TRUE;

    return TRUE;
}





// function: disable SATA IRQ
// 1 find SATA device
// 2 move to address: 0x3c
// 3 disable IRQ ([0x3c]=0xff)
void DisableSataIrq()
{
	ULONG  PCIAddr;
	ULONG  PCIData;
	ULONG  gbFunNum, gbBusNum, gbDevNum;

	for (gbFunNum=0; gbFunNum<8/*MAX_PCI_FUN*/; gbFunNum++) {
		for (gbBusNum=0; gbBusNum<3/*MAX_PCI_BUS*/; gbBusNum++) {
			for (gbDevNum=0; gbDevNum<32/*MAX_PCI_DEVICE*/; gbDevNum++) {
				PCIAddr=0x80000000+(gbBusNum<<16)+(gbDevNum<<11)+(gbFunNum<<8);
				__asm {
					mov  dx, 0cf8h
					mov  eax, PCIAddr
					out  dx, eax
					mov  dx, 0cfch
					in   eax, dx
					mov  PCIData, eax
				}

				#define VT8237R_SATA_ID 0x31491106
				if (PCIData==VT8237R_SATA_ID) {
					RETAILMSG(1, (TEXT("Disable VIA SATA IRQ\r\n")));
					#define REG_INT_LINE 0x3C
					PCIAddr += REG_INT_LINE;
					__asm {
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						mov  eax, 0xff
						out  dx, eax
					}
					return;
				}
			}
		}
	}
}

void DisableEHCILegacy()
{
    ULONG  PCIAddr;
    ULONG  PCIData;
    ULONG  gbFunNum, gbBusNum, gbDevNum;
    
    BOOL   bEHCI = FALSE;
	for (gbFunNum=0; gbFunNum<8/*MAX_PCI_FUN*/; gbFunNum++)
	{
	   for (gbBusNum=0; gbBusNum<3/*MAX_PCI_BUS*/; gbBusNum++)
	   {
	      for (gbDevNum=0; gbDevNum<32/*MAX_PCI_DEVICE*/; gbDevNum++)
	      {
		 PCIAddr=0x80000000+(gbBusNum<<16)+(gbDevNum<<11)+(gbFunNum<<8);
		 __asm
		 {
		      mov  dx, 0cf8h
	              mov  eax, PCIAddr
	              out  dx, eax
	              mov  dx, 0cfch
	              in   eax, dx
	              mov  PCIData, eax
	         }
	         
	         if (PCIData==0x31041106)
	         {
	         	RETAILMSG(1, (TEXT("Disable VIA EHCI Legacy\r\n")));
	         	PCIAddr += 0x6C;
		         __asm
            		{
                 		mov  dx, 0cf8h
                 		mov  eax, PCIAddr
                 		out  dx, eax
                 		mov  dx, 0cfch
                 		mov  eax, 0
                 		out  dx, eax
            		}
		      bEHCI = TRUE;
            		break;   
	         
	         }
	      }
	      if (bEHCI)
                 break;
	
	    }
	    if (bEHCI)
                 break;
	}
}

void DisableUHCILegacy()
{
	ULONG  PCIAddr;
	ULONG  PCIData;
	ULONG  gbFunNum, gbBusNum, gbDevNum;
	USHORT usUSBIoBase, usUSBStatus;
	BOOL   bUHCI = FALSE;
	int i;

	for (gbFunNum=0; gbFunNum<8/*MAX_PCI_FUN*/; gbFunNum++)
	{
		for (gbBusNum=0; gbBusNum<3/*MAX_PCI_BUS*/; gbBusNum++)
		{
			for (gbDevNum=0; gbDevNum<32/*MAX_PCI_DEVICE*/; gbDevNum++)
			{
				PCIAddr=0x80000000+(gbBusNum<<16)+(gbDevNum<<11)+(gbFunNum<<8);
				__asm
				{
					mov  dx, 0cf8h
					mov  eax, PCIAddr
					out  dx, eax
					mov  dx, 0cfch
					in   eax, dx
					mov  PCIData, eax
				}
				
				if (PCIData==0x30381106)
				{
					RETAILMSG(1, (TEXT("Disable VIA UHCI Legacy\r\n")));

					DEBUGMSG(1, (TEXT("Find VIA USB Device\r\n")));
					// Get USB IO Address, and Stop USB Controller
					PCIAddr += 0x20;
					__asm
					{
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						in   eax, dx
						and  ax, 0fffeh
						mov  usUSBIoBase, ax
						mov  dx, ax
						xor  ax, ax
						out  dx, ax
					}
					DEBUGMSG(1, (TEXT("VIA USB IoBase %x\r\n"), usUSBIoBase));

					// Check USB Halt
					for ( i = 0; i < 0x1000; i++)
					{
						__asm
						{
							mov  ax, usUSBIoBase
							add  ax, 2
							mov  dx, ax
							in   ax, dx
							mov  usUSBStatus, ax
							out  dx, ax
						}

						if (usUSBStatus & 0x20)
							break;
					}

					DEBUGMSG(1, (TEXT("VIA USB Status %x, Loop %d\r\n"), usUSBStatus, i));

					// Disable SMI for USB
					PCIAddr += 0xa0;
					__asm
					{
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						mov  eax, 2000h
						out  dx, eax
					}

					PCIAddr += 0x60;
					__asm
					{
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						in   eax, dx
						and  ax, 0fffeh
						mov  usUSBIoBase, ax
						mov  dx, ax
						xor  ax, ax
						out  dx, ax
					}

					for ( i = 0; i < 0x1000; i++)
					{
						__asm
						{
							mov  ax, usUSBIoBase
							add  ax, 2
							mov  dx, ax
							in   ax, dx
							mov  usUSBStatus, ax
							out  dx, ax
						}

						if (usUSBStatus & 0x20)
							break;
					}
					DEBUGMSG(1, (TEXT("VIA USB2 IoBase %x\r\n"), usUSBIoBase));
					DEBUGMSG(1, (TEXT("VIA USB Status %x, Loop %d\r\n"), usUSBStatus, i));

					PCIAddr += 0xa0;
					__asm
					{
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						mov  eax, 2000h
						out  dx, eax
					}

					// Check 8235
					PCIAddr += 0x100;
					PCIAddr -= 0xc0;
					__asm
					{
						mov  dx, 0cf8h
						mov  eax, PCIAddr
						out  dx, eax
						mov  dx, 0cfch
						in   eax, dx
						mov  PCIData, eax
					}

					if (PCIData==0x30381106)
					{
						// For VT8235
						PCIAddr += 0x20;
						__asm
						{
							mov  dx, 0cf8h
							mov  eax, PCIAddr
							out  dx, eax
							mov  dx, 0cfch
							in   eax, dx
							and  ax, 0fffeh
							mov  usUSBIoBase, ax
							mov  dx, ax
							xor  ax, ax
							out  dx, ax
						}

						for ( i = 0; i < 0x1000; i++)
						{
							__asm
							{
								mov  ax, usUSBIoBase
								add  ax, 2
								mov  dx, ax
								in   ax, dx
								mov  usUSBStatus, ax
								out  dx, ax
							}

							if (usUSBStatus & 0x20)
								break;
						}
						DEBUGMSG(1, (TEXT("VIA USB3 IoBase %x\r\n"), usUSBIoBase));
						DEBUGMSG(1, (TEXT("VIA USB Status %x, Loop %d\r\n"), usUSBStatus, i));

						PCIAddr += 0xa0;
						__asm
						{
							mov  dx, 0cf8h
							mov  eax, PCIAddr
							out  dx, eax
							mov  dx, 0cfch
							mov  eax, 2000h
							out  dx, eax
						}

						// Check 8237
						PCIAddr += 0x100;
						PCIAddr -= 0xc0;
						__asm
						{
							mov  dx, 0cf8h
							mov  eax, PCIAddr
							out  dx, eax
							mov  dx, 0cfch
							in   eax, dx
							mov  PCIData, eax
						}

						if (PCIData==0x30381106)
						{
							// For VT8237
							PCIAddr += 0x20;
							__asm
							{
								mov  dx, 0cf8h
								mov  eax, PCIAddr
								out  dx, eax
								mov  dx, 0cfch
								in   eax, dx
								and  ax, 0fffeh
								mov  usUSBIoBase, ax
								mov  dx, ax
								xor  ax, ax
								out  dx, ax
							}

							for ( i = 0; i < 0x1000; i++)
							{
								__asm
								{
									mov  ax, usUSBIoBase
									add  ax, 2
									mov  dx, ax
									in   ax, dx
									mov  usUSBStatus, ax
									out  dx, ax
								}

								if (usUSBStatus & 0x20)
									break;
							}
							DEBUGMSG(1, (TEXT("VIA USB4 IoBase %x\r\n"), usUSBIoBase));
							DEBUGMSG(1, (TEXT("VIA USB Status %x, Loop %d\r\n"), usUSBStatus, i));

							PCIAddr += 0xa0;
							__asm
							{
								mov  dx, 0cf8h
								mov  eax, PCIAddr
								out  dx, eax
								mov  dx, 0cfch
								mov  eax, 2000h
								out  dx, eax
							}
						}
					}

					bUHCI = TRUE;
					break;
				}
			}

			if (bUHCI)
				break;
		}

		if (bUHCI)
			break;
	}

}